
showdown   = require 'showdown'
markdown   = new showdown.converter()
co         = require 'co'
_          = require 'lodash'
path       = require 'path'
handlebars = require 'handlebars'

Methods    = require './methods'
methods    = new Methods()


Templates  = require './templates'
templates  = new Templates()

Config     = require './config'
config     = new Config()

Menu       = require './menu'
menu       = new Menu()

class Parser

  #===============================================================#
  #                 PRIVATE METHOD __metaToObject                 #
  #===============================================================#
  ###*
   * Converts markdown metaString to an array of objects
   * @param  {[String]} metaString [Meta text from markdown file]
   * @return {[Array]}  [Array of object]
  ###
  #===============================================================#
  __metaToObject = (metaString) ->
    if metaString
      meta = _.compact(metaString.split "\n")

      # read all lines as an array and make
      # key/value pairs using : as delimeter
      _.zipObject (
        _.compact (
          _.map meta, (val) ->
            [key,value]   =   val.split ":"
            if                value
            then              [key.trim(),value.trim()]
        )
      )








  #===============================================================#
  #               PUBLIC METHOD __extractMetaContent             #
  #===============================================================#
  ###*
   * Extracting meta content , from markdown file text
   * @param  {[String]} markDownText [markdown file content as valid utf-8 string]
   * @return {[Array]}              [returns meta object and html using showdown]
  ###
  #===============================================================#
  __extractMetaContent : (markDownText) ->
  
    if not markDownText
      throw "valid markdown text missing"
    
    # if first 3 letters are ---
    if markDownText[0...3] is '---'
      meta    = markDownText.replace /^-{3,}\s([\s\S]*?)-{3,}(\s[\s\S]*|\s?)$/,"$1"

    # setting up regex to remove meta string and blank spaces from markdown.
    if        _.size meta
    then      regex   = new RegExp "#{meta}(?:.|s)|-|/ /|","g"
    else      regex   = new RegExp "-|/ /|","g"      

    content    = markDownText.replace regex,""
    
    ## getting meta object from string
    metaObject = __metaToObject(meta)

    ## setting meta object to empty object if undefined
    metaObject = {} if typeof(metaObject) is 'undefined'

    ## Converting final Markdown to html using showdown
    content    = markdown.makeHtml(content.trim())

    ## returning array
    [metaObject,content]








  #===============================================================#
  #                PRIVATE METHOD _getContent                     #
  #===============================================================#
  ###
   * Read file using readFile Thunks
   * @param  {[string]} filePath
   * @return {[buffer]}          [file buffer]
  ###
  #===============================================================#
  _getContent = (filePath) =>
    yield methods.readFile filePath








  #===============================================================#
  #               PRIVATE METHOD __registerPartials               #
  #===============================================================#
  __registerPartials = (partials) =>
    if _.size partials
      yield _.map partials,(partial,key) ->
        handlebars.registerPartial key,partial.toString('utf-8')



  __registerHelpers = (helpers) =>
    if _.size helpers
      yield _.map helpers,(helper,key) ->
        handlebars.registerHelper key,helper



  _menuToHtml = (menu,outpur_dir) ->
    _.map menu, (items,key) ->
      _.map items.childs, (child) ->
        if key isnt 'welcome'
          child.url = "#{outpur_dir}/#{key}/#{child.slug}.html"
        else
          child.url = "#{outpur_dir}/#{child.slug}.html"        
      items


  #===============================================================#
  #                PUBLIC METHOD (buildHtml)                      #
  #===============================================================# 
  ###*
   * Public method to build html file from markdown
   * All hardwork done
   * @param  {[String]} markDownPath [Path to markdown file]
  ###
  #===============================================================#
  buildHtml : co.wrap (markDownPath,menuObject,configObject,helpersList) ->
        
    yield __registerHelpers(helpersList)

    ###*
      * Reading all templates inside templates directory
      * and storing them as key/value pairs using Templates Class
    ###
          
    templatesPath    = methods.buildPath configObject.templates_dir,methods.basePath
    templatesList    = yield templates.getTemplates templatesPath
    
    ###*
      * Reading all partials inside partials directory
      * and storing them as key/value pairs using Templates Class
    ###
    partialsPaths    = methods.buildPath configObject.partials_dir,methods.basePath
    partials         = yield templates.getPartials partialsPaths
            
    ###*
      * registering partials with handlebars, so they ready to use
    ###
    yield __registerPartials(partials)

    ###*
      * Getting markdown file content using markdown path
    ###
    fileContent    = yield _getContent markDownPath

    ###*
      * Extracting metaData and content body from markdown file.
      * In case of no meta data, meta will be an empty object and
      * defaults should be used for same
      
      ===========================
              META DEFAULTS
      ===========================
      - template [defaultsTo][index]
    ###
    [meta,content] = @__extractMetaContent(fileContent.toString())
    
    template = typeof meta.template isnt 'undefined' ? meta.template.replace(/["'"]+/g, '') || 'index'

    ###*
      * Grabbing template from templates list , and converting
      * buffer to valid utf-8 string
    ###
    
    template = "#{template}.handlebars"
        
    if templatesList[template]
      templateHtml   = templatesList[template].toString 'utf-8'
    else
      throw "Template #{template} does not exists"
    
    ###*
      * Making menu ready to be consumed by templates
    ###
    menu = _menuToHtml menuObject,configObject.output_dir

    ###*
      * Compiling template using handlebars
    ###
    template       = handlebars.compile templateHtml
    
    context =
      content : content
      config  : configObject
      meta    : meta
      menu    : menu

    template context

module.exports = Parser